//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, April-2009
//-----------------------------------------------------------------------------
//
// RandomParticle.h
// C++ interface to F77 functions: random particle
//-----------------------------------------------------------------------------
//*****************************************************************************

#ifndef _RandomParticle_h
#define _RandomParticle_h

class RandomParticle
{
 public:
  // constructors
  RandomParticle();
  RandomParticle(double p, double thetaDeg, double phiDeg, int LUND=211);
  // selectors
  double X(){return m_X;} 
  double Y(){return m_Y;} 
  double Z(){return m_Z;} 
  double Tstart(){return m_Tstart;} 
  double Tstop(){return m_Tstop;}
  double Px(){return m_Px;} 
  double Py(){return m_Py;} 
  double Pz(){return m_Pz;} 
  double p();
  double theta();
  double phi();
  int Lund(){return m_LUND;} 
  int Label(){return m_REF;}
  int Q();
  int Hyp();
  // modifiers
  void setPmin(double pmin); // set minimal momentum
  void setPmax(double pmax); // set maximal momentum
  void setCosThmin(double cthmin); // set minimal cos(theta)
  void setCosThmax(double cthmax); // set maximal cos(theta)
  void setFrac(double e, double mu, double pi, double K, double p); // set fractions
  void genNext(); // generate next particle
  // print to std output
  void Dump();

 private:
  double m_X; 
  double m_Y; 
  double m_Z; 
  double m_Tstart; 
  double m_Tstop;
  double m_Px; 
  double m_Py; 
  double m_Pz; 
  int m_LUND; 
  int m_REF;
  double m_pmin;
  double m_pmax;
  double m_cthmin;
  double m_cthmax;
  double m_frac[5];
  double pi;
};

#endif

